@@ -41,7 +41,7 @@ programmatically.  A document can have multiple indexes by adding extra
 import pymongo
 from pymongo import GEO2D
 from collections import defaultdict
-from mongoalchemy.util import classproperty
+from mongoalchemy.util import classproperty, UNSET
 from mongoalchemy.query_expression import QueryField
 from mongoalchemy.fields import ObjectIdField, Field, BadValueException, SCALAR_MODIFIERS
 from mongoalchemy.exceptions import DocumentException, MissingValueException, ExtraValueException, FieldNotRetrieved, BadFieldSpecification
@@ -51,8 +51,10 @@ document_type_registry = defaultdict(dict)
 class DocumentMeta(type):
     def __new__(mcs, classname, bases, class_dict):
         # Validate Config Options
+        # print '-' * 20, classname, '-' * 20
         
         # Create Class
+        class_dict['_subclasses'] = {}
         new_class = type.__new__(mcs, classname, bases, class_dict)
         
         if new_class.config_extra_fields not in ['error', 'ignore']:
@@ -73,11 +75,17 @@ class DocumentMeta(type):
         
         # 2. create a dict of fields to set on the object
         new_class._fields = {}
-        for name in dir(new_class):
-            field = getattr(new_class, name)
-            if not isinstance(field, QueryField):
+        for b in bases:
+            # print b
+            if not hasattr(b, 'get_fields'):
                 continue
-            new_class._fields[name] = field.get_type()
+            for name, field in b.get_fields().iteritems():    
+                new_class._fields[name] = field
+
+        for name, maybefield in class_dict.iteritems():
+            if not isinstance(maybefield, Field):
+                continue
+            new_class._fields[name] = maybefield
         
         # 3. register type
         if new_class.config_namespace != None:
@@ -85,7 +93,25 @@ class DocumentMeta(type):
             if name == None:
                 name = new_class.__name__
             document_type_registry[new_class.config_namespace][name] = new_class
+
+        # 5. Add proxies
+        for name, field in new_class.get_fields().iteritems():
+            if field.proxy is not None:
+                setattr(new_class, field.proxy, Proxy(name))
+            if field.iproxy is not None:
+                setattr(new_class, field.iproxy, IProxy(name, field.ignore_missing))
         
+        # 4.  Add subclasses
+
+        for b in bases:
+            if 'Document' in globals() and issubclass(b, Document):
+                b.add_subclass(new_class)
+            if not hasattr(b, 'config_polymorphic_collection'):
+                continue
+            if b.config_polymorphic_collection and 'config_collection' not in class_dict:
+                new_class.config_collection_name = b.get_collection_name()
+
+
         return new_class
 
 class Document(object):
@@ -109,6 +135,24 @@ class Document(object):
         "global"
     '''
     
+    config_polymorphic = None
+    ''' The variable to use when determining which class to instantiate a
+        database object with.  It is the name of an attribute which
+        will be used to decide the type of the object.  If you want more 
+        control over which class is selected, you can override 
+        ``get_subclass``.
+    '''
+    
+    config_polymorphic_collection = False
+    ''' Use the base class collection name for the subclasses.  Default: False
+    '''
+
+    config_polymorphic_identity = None
+    ''' When using a string value with ``config_polymorphic_on`` in a parent 
+        class, this is the value that the attribute is compared to when 
+        determining 
+    '''
+    
     config_full_name = None
     ''' If namespaces are being used, the key for a class is normally
         the class name.  In some cases the same class name may be used in 
@@ -122,7 +166,7 @@ class Document(object):
         document constructor.  Possible values are 'error' and 'ignore'. Any 
         fields which couldn't be mapped can be retrieved (and edited) using
         :func:`~Document.get_extra_fields` '''
-    
+
     def __init__(self, retrieved_fields=None, loading_from_db=False, **kwargs):
         ''' :param retrieved_fields: The names of the fields returned when loading \
                 a partial object.  This argument should not be explicitly set \
@@ -132,7 +176,7 @@ class Document(object):
                 any missing (but required) fields will raise a :class:`~mongoalchemy.document.MissingValueException`. \
                 Both types of exceptions are subclasses of :class:`~mongoalchemy.document.DocumentException`.
         '''
-        self.partial = retrieved_fields != None
+        self.partial = retrieved_fields is not None
         self.retrieved_fields = self.__normalize(retrieved_fields)
         
         self._dirty = {}
@@ -150,6 +194,10 @@ class Document(object):
             if name in kwargs:
                 getattr(cls, name).set_value(self, kwargs[name], from_db=loading_from_db)
                 continue
+            # DO I NEED THIS?
+            # elif field.default != UNSET:
+            #     getattr(cls, name).set_value(self, field.default, from_db=loading_from_db)
+
         
         for k in kwargs:
             if k not in fields:
@@ -160,6 +208,52 @@ class Document(object):
 
         self.__extra_fields_orig = dict(self.__extra_fields)
     
+    def __deepcopy__(self, memo):
+        return type(self).unwrap(self.wrap(), session=self._get_session())
+
+    @classmethod
+    def add_subclass(cls, subclass):
+        ''' Register a subclass of this class.  Maps the subclass to the 
+            value of subclass.config_polymorphic_identity if available.
+        '''
+        # if not polymorphic, stop
+        if hasattr(subclass, 'config_polymorphic_identity'):
+            attr = subclass.config_polymorphic_identity
+            cls._subclasses[attr] = subclass
+
+
+    @classmethod
+    def get_subclass(cls, obj):
+        ''' Get the subclass to use when instantiating a polymorphic object.
+            The default implementation looks at ``cls.config_polymorphic``
+            to get the name of an attribute.  Subclasses automatically 
+            register their value for that attribute on creation via their
+            ``config_polymorphic_identity`` field.  This process is then 
+            repeated recursively until None is returned (indicating that the
+            current class is the correct one)
+
+            This method can be overridden to allow any method you would like 
+            to use to select subclasses. It should return either the subclass
+            to use or None, if the original class should be used.
+        '''
+        if cls.config_polymorphic is None:
+            return
+
+        value = obj.get(cls.config_polymorphic)
+        value = cls._subclasses.get(value)
+        if value == cls or value is None:
+            return None
+
+        sub_value = value.get_subclass(obj)
+        if sub_value is None:
+            return value
+        return sub_value
+    def __eq__(self, other):
+        try:
+            return self.mongo_id == other.mongo_id
+        except:
+            return False
+
     def get_dirty_ops(self, with_required=False):
         ''' Returns a dict with the update operations necessary to make the 
             changes to this object to the database version.  It is mainly used
@@ -269,18 +363,6 @@ class Document(object):
             return False
         return True 
     
-    def commit(self, db, safe=True):
-        ''' Save this object to the database and set the ``_id`` field of this
-            document to the returned id.
-            
-            :param db: The pymongo database to write to
-        '''
-        collection = db[self.get_collection_name()]
-        for index in self.get_indexes():
-            index.ensure(collection)
-        id = collection.save(self.wrap(), safe=safe)
-        self.mongo_id = id
-    
     def wrap(self):
         ''' Returns a transformation of this document into a form suitable to 
             be saved into a mongo database.  This is done by using the ``wrap()``
@@ -289,31 +371,33 @@ class Document(object):
         for k, v in self.__extra_fields.iteritems():
             res[k] = v
         cls = self.__class__
-        for name in dir(cls):
+        for name in self.get_fields():
             field = getattr(cls, name)
-            if not isinstance(field, QueryField):
-                continue
+            # if not isinstance(field, QueryField):
+            #     continue
             try:
                 value = getattr(self, name)
                 res[field.db_field] = field.wrap(value)
-            except AttributeError:
+            except AttributeError, e:
                 if field.required:
                     raise MissingValueException(name)
         return res
     
     @classmethod
-    def validate_unwrap(cls, obj, fields=None):
+    def validate_unwrap(cls, obj, fields=None, session=None):
         ''' Attempts to unwrap the document, and raises a BadValueException if
             the operation fails. A TODO is to make this function do the checks
             without actually doing the (potentially expensive) 
             deserialization'''
         try:
-            cls.unwrap(obj, fields=fields)
+            cls.unwrap(obj, fields=fields, session=session)
         except Exception, e:
-            raise BadValueException('Document', obj, 'Exception validating document', cause=e)
+            e_type = type(e).__name__
+            cls_name = cls.__name__
+            raise BadValueException(cls_name, obj, '%s Exception validating document' % e_type, cause=e)
     
     @classmethod
-    def unwrap(cls, obj, fields=None):
+    def unwrap(cls, obj, fields=None, session=None):
         ''' Returns an instance of this document class based on the mongo object 
             ``obj``.  This is done by using the ``unwrap()`` methods of the 
             underlying fields to set values.
@@ -323,12 +407,16 @@ class Document(object):
                     for the fields to load.  If ``None`` is passed all fields  \
                     are loaded
             '''
-        
+
+        subclass = cls.get_subclass(obj)
+        if subclass and subclass != cls:
+            unwrapped = subclass.unwrap(obj, fields=fields, session=session)
+            unwrapped._session = session
+            return unwrapped
         # Get reverse name mapping
         name_reverse = {}
         for name, field in cls.get_fields().iteritems():
             name_reverse[field.db_field] = name
-        
         # Unwrap
         params = {}
         for k, v in obj.iteritems():
@@ -336,20 +424,34 @@ class Document(object):
             if not hasattr(cls, k) and cls.config_extra_fields:
                 params[str(k)] = v
                 continue
+
             field = getattr(cls, k)
-            if fields != None and isinstance(field.get_type(), DocumentField):
+            field_is_doc = fields is not None and isinstance(field.get_type(), DocumentField)
+
+            extra_unwrap = {}
+            if field.has_autoload:
+                extra_unwrap['session'] = session
+            if field_is_doc:
                 normalized_fields = cls.__normalize(fields)
-                unwrapped = field.unwrap(v, fields=normalized_fields.get(k))
+                unwrapped = field.unwrap(v, fields=normalized_fields.get(k), **extra_unwrap)
             else:
-                unwrapped = field.unwrap(v)
+                unwrapped = field.unwrap(v, **extra_unwrap)
+            unwrapped = field.localize(session, unwrapped)
             params[str(k)] = unwrapped
         
-        if fields != None:
+        if fields is not None:
             params['retrieved_fields'] = fields
         obj = cls(loading_from_db=True, **params)
         obj.__mark_clean()
+        obj._session = session
         return obj
-    
+
+    _session = None
+    def _get_session(self):
+        return self._session
+    def _set_session(self, session):
+        self._session = session
+
     def __mark_clean(self):
         self._dirty.clear()
         
@@ -398,6 +500,7 @@ class DocumentField(Field):
     ''' A field which wraps a :class:`Document`'''
     
     has_subfields = True
+    has_autoload = True
     
     def __init__(self, document_class, **kwargs):
         super(DocumentField, self).__init__(**kwargs)
@@ -464,26 +567,26 @@ class DocumentField(Field):
         self.validate_wrap(value)
         return self.type.wrap(value)
     
-    def unwrap(self, value, fields=None):
+    def unwrap(self, value, fields=None, session=None):
         ''' Validate ``value`` and then use the document's class to unwrap the 
             value'''
-        self.validate_unwrap(value, fields=fields)
-        return self.type.unwrap(value, fields=fields)
+        self.validate_unwrap(value, fields=fields, session=session)
+        return self.type.unwrap(value, fields=fields, session=session)
     
     def validate_wrap(self, value):
         ''' Checks that ``value`` is an instance of ``DocumentField.type``.
             if it is, then validation on its fields has already been done and
             no further validation is needed.
         '''
-        if value.__class__ != self.type:
+        if not isinstance(value, self.type):
             self._fail_validation_type(value, self.type)
     
-    def validate_unwrap(self, value, fields=None):
+    def validate_unwrap(self, value, fields=None, session=None):
         ''' Validates every field in the underlying document type.  If ``fields`` 
             is not ``None``, only the fields in ``fields`` will be checked.
         '''
         try:
-            self.type.validate_unwrap(value, fields=fields)
+            self.type.validate_unwrap(value, fields=fields, session=session)
         except BadValueException, bve:
             self._fail_validation(value, 'Bad value for DocumentField field', cause=bve)
 
@@ -586,3 +689,36 @@ class Index(object):
             drop_dups=self.__drop_dups, **extras)
         return self
         
+class Proxy(object):
+    def __init__(self, name):
+        self.name = name
+    def __get__(self, instance, owner):
+        if instance is None:
+            return getattr(owner, self.name)
+        session = instance._get_session()
+        ref = getattr(instance, self.name)
+        if ref is None:
+            return None
+        return session.dereference(ref)
+    def __set__(self, instance, value):
+        assert instance is not None
+        setattr(instance, self.name, value)
+
+class IProxy(object):
+    def __init__(self, name, ignore_missing):
+        self.name = name
+        self.ignore_missing = ignore_missing
+    def __get__(self, instance, owner):
+        if instance is None:
+            return getattr(owner, self.name)
+        session = instance._get_session()
+        def iterator():
+            for v in getattr(instance, self.name):
+                if v is None:
+                    yield v
+                    continue
+                value = session.dereference(v, allow_none=self.ignore_missing)
+                if value is None and self.ignore_missing:
+                    continue
+                yield value
+        return iterator()
