@@ -70,7 +70,7 @@ class Query(object):
         return flatten(self.__query)
     
     def __get_query_result(self):
-        return self.session.execute_query(self)
+        return self.session.execute_query(self, self.session)
     
     def raw_output(self):
         self._raw_output = True
@@ -374,31 +374,45 @@ class Query(object):
         ''' Refer to:  :func:`~mongoalchemy.update_expression.UpdateExpression.pop_last`'''
         return UpdateExpression(self).pop_last(qfield)
 
-
 class QueryResult(object):
-    def __init__(self, cursor, type, raw_output=False, fields=None):
+    def __init__(self, session, cursor, type, raw_output=False, fields=None):
         self.cursor = cursor
         self.type = type
         self.fields = fields
         self.raw_output = raw_output
+        self.session = session
     
     def next(self):
         value = self.cursor.next()
         if not self.raw_output:
-            value = self.type.unwrap(value, fields=self.fields)
+            db = self.cursor.collection.database
+            conn = db.connection
+            obj = self.session.cache_read(value['_id'])
+            if obj:
+                return obj
+            value = self.type.unwrap(value, fields=self.fields, session=self.session)
+            if not isinstance(value, dict):
+                self.session.cache_write(value)
         return value
     
     def __getitem__(self, index):
         value = self.cursor.__getitem__(index)
         if not self.raw_output:
-            value = self.type.unwrap(value)
+            db = self.cursor.collection.database
+            conn = db.connection
+            obj = self.session.cache_read(value['_id'])
+            if obj:
+                return obj
+            value = self.type.unwrap(value, session=self.session)
+            self.session.cache_write(value)
+            # value = self.session.localize(session, value)
         return value
     
     def rewind(self):
         return self.cursor.rewind()
     
     def clone(self):
-        return QueryResult(self.cursor.clone(), self.type, 
+        return QueryResult(self.session, self.cursor.clone(), self.type, 
             raw_output=self.raw_output, fields=self.fields)
     
     def __iter__(self):
