@@ -41,16 +41,18 @@
     anything intelligent for ordering.
 '''
 
-
+from uuid import uuid4
 from pymongo.connection import Connection
+from bson import DBRef, ObjectId
 from mongoalchemy.query import Query, QueryResult, RemoveQuery
-from mongoalchemy.document import FieldNotRetrieved
+from mongoalchemy.document import FieldNotRetrieved, Document
 from mongoalchemy.query_expression import FreeFormDoc
-from itertools import chain
+from mongoalchemy.exceptions import TransactionException
+from mongoalchemy.ops import *
 
 class Session(object):
 
-    def __init__(self, database, safe=False):
+    def __init__(self, database, tz_aware=False, timezone=None, safe=False, cache_size=0):
         '''
         Create a session connecting to `database`.  
         
@@ -63,13 +65,30 @@ class Session(object):
             * db: the underlying pymongo database object
             * queue: the queue of unflushed database commands (currently useless \
                 since there aren't any operations which defer flushing)
+            * cache_size: The size of the identity map to keep.  When objects \
+                            are pulled from the DB they are checked against this \
+                            map and if present, the existing object is used.  \
+                            Defaults to 0, use None to only clear at session end.
+
         '''
         self.db = database
         self.queue = []
         self.safe = safe
+        self.timezone = timezone
+        self.tz_aware = bool(tz_aware or timezone)
+
+        self.cache_size = cache_size
+        self.cache = {}
+        self.transactions = []
+    @property
+    def autoflush(self):
+        return not self.in_transaction
+    @property
+    def in_transaction(self):
+        return len(self.transactions) > 0
     
     @classmethod
-    def connect(self, database, *args, **kwds):
+    def connect(self, database, timezone=None, cache_size=0, *args, **kwds):
         ''' `connect` is a thin wrapper around __init__ which creates the 
             database connection that the session will use.
             
@@ -83,24 +102,61 @@ class Session(object):
         safe = kwds.get('safe', False)
         if 'safe' in kwds:
             del kwds['safe']
+        if timezone is not None:
+            kwds['tz_aware'] = True
         conn = Connection(*args, **kwds)
         db = conn[database]
-        return Session(db, safe=safe)
+        return Session(db, timezone=timezone, safe=safe, cache_size=cache_size)
     
+    def cache_write(self, obj, mongo_id=None):
+        if mongo_id is None:
+            mongo_id = obj.mongo_id
+
+        if self.cache_size == 0:
+            return
+        if mongo_id in self.cache:
+            return
+        if self.cache_size is not None and len(self.cache) >= self.cache_size:
+            for key in self.cache:
+                break
+            del self.cache[key]
+        assert isinstance(mongo_id, ObjectId), 'Currently, cached objects must use mongo_id as an ObjectId.  Got: %s' % type(mongo_id)
+        self.cache[mongo_id] = obj
+
+    def cache_read(self, id):
+        if self.cache_size == 0:
+            return
+        assert isinstance(id, ObjectId), 'Currently, cached objects must use mongo_id as an ObjectId'
+        # if not isinstance(id, ObjectId):
+        #     id = ObjectId(id)
+        if id in self.cache:
+            return self.cache[id]
+        return None
+
     def end(self):
         ''' End the session.  Flush all pending operations and ending the 
             *pymongo* request'''
-        self.flush()
+        self.cache = {}
+        if not self.transactions:
+            self.flush()
         self.db.connection.end_request()
     
     def insert(self, item, safe=None):
-        ''' Insert an item into the queue and flushes.  Later this function should be smart and delay 
-            insertion until the _id field is actually accessed'''
+        ''' Insert an item into the work queue and flushes.'''
+        self.add(item, safe=safe)
+    
+    def add(self, item, safe=None):
+        ''' Add an item into the queue of things to be inserted.  Does not flush.'''
+        item._set_session(self)
         if safe is None:
             safe = self.safe
-        self.queue.append(item)
-        self.flush(safe=safe)
-    
+        self.queue.append(SaveOp(self.transaction_id, self, item, safe))
+        # after the save op is recorded, the document has an _id and can be 
+        # cached
+        self.cache_write(item)
+        if self.autoflush:
+            return self.flush()
+
     def update(self, item, id_expression=None, upsert=False, update_ops={}, safe=None, **kwargs):
         ''' Update an item in the database.  Uses the on_update keyword to each
             field to decide which operations to do, or.  
@@ -127,27 +183,13 @@ class Session(object):
                 This operation is **experimental** and **not fully tested**,
                 although it does have code coverage.  
             '''
-        if id_expression:
-            db_key = Query(type(item), self).filter(id_expression).query
-        else:
-            db_key = {'_id' : item.mongo_id}
-
-        dirty_ops = item.get_dirty_ops(with_required=upsert)
-        for key, op in chain(update_ops.items(), kwargs.items()):
-            key = str(key)
-            for current_op, keys in dirty_ops.items():
-                if key not in keys:
-                    continue
-                dirty_ops.setdefault(op,{})[key] = keys[key]
-                del dirty_ops[current_op][key]
-                if len(dirty_ops[current_op]) == 0:
-                    del dirty_ops[current_op]
         if safe is None:
             safe = self.safe
-        self.flush(safe=safe)
-        self.db[item.get_collection_name()].update(db_key, dirty_ops, upsert=upsert, safe=safe)
+        self.queue.append(UpdateDocumentOp(self.transaction_id, self, item, safe, id_expression=id_expression, 
+                          upsert=upsert, update_ops=update_ops, **kwargs))
+        if self.autoflush:
+            return self.flush()
         
-    
     def query(self, type):
         ''' Begin a query on the database's collection for `type`.  If `type`
             is an instance of basesting, the query will be in raw query mode
@@ -162,16 +204,21 @@ class Session(object):
             type = FreeFormDoc(type)
         return Query(type, self)
     
-    def execute_query(self, query):
-        ''' Get the results of ``query``.  This method will flush the queue '''
+    def add_to_session(self, obj):
+        obj._set_session(self)
+    
+    def execute_query(self, query, session):
+        ''' Get the results of ``query``.  This method does flush in a 
+            transaction, so any objects retrieved which are not in the cache
+            which would be updated when the transaction finishes will be 
+            stale '''
         collection = self.db[query.type.get_collection_name()]
         for index in query.type.get_indexes():
             index.ensure(collection)
-        
+
         kwargs = dict()
         if query.get_fields():
             kwargs['fields'] = [str(f) for f in query.get_fields()]
-        
         cursor = collection.find(query.query, **kwargs)
         
         if query.sort:
@@ -182,7 +229,7 @@ class Session(object):
             cursor.limit(query.get_limit())
         if query.get_skip() != None:
             cursor.skip(query.get_skip())
-        return QueryResult(cursor, query.type, raw_output=query._raw_output, fields=query.get_fields())
+        return QueryResult(session, cursor, query.type, raw_output=query._raw_output, fields=query.get_fields())
     
     def remove_query(self, type):
         ''' Begin a remove query on the database's collection for `type`.
@@ -201,48 +248,42 @@ class Session(object):
             :param safe: whether to wait for the operation to complete.  Defaults \
                 to the session's ``safe`` value.
         '''
-        self.flush()
         if safe is None:
             safe = self.safe
-        if not obj.has_id():
-            return None
-        collection = self.db[obj.get_collection_name()]
-        for index in obj.get_indexes():
-            index.ensure(collection)
-        return self.db[obj.get_collection_name()].remove(obj.mongo_id, safe=safe)
+        remove = RemoveDocumentOp(self.transaction_id, self, obj, safe)
+        self.queue.append(remove)
+        if self.autoflush:
+            return self.flush()
     
     def execute_remove(self, remove):
         ''' Execute a remove expression.  Should generally only be called implicitly.
         '''
-        self.flush()
+
         safe = self.safe
-        if remove.safe != None:
+        if remove.safe is not None:
             safe = remove.safe
         
-        collection = self.db[remove.type.get_collection_name()]
-        for index in remove.type.get_indexes():
-            index.ensure(collection)
-
-        return self.db[remove.type.get_collection_name()].remove(remove.query, safe=safe)
+        self.queue.append(RemoveOp(self.transaction_id, self, remove.type, safe, remove))
+        if self.autoflush:
+            return self.flush()
     
     def execute_update(self, update, safe=False):
         ''' Execute an update expression.  Should generally only be called implicitly.
         '''
         
-        self.flush()
+        # safe = self.safe
+        # if update.safe is not None:
+        #     safe = remove.safe
+
         assert len(update.update_data) > 0
-        collection = self.db[update.query.type.get_collection_name()]
-        for index in update.query.type.get_indexes():
-            index.ensure(collection)
-        kwargs = dict(
-            upsert=update.get_upsert(), 
-            multi=update.get_multi(),
-            safe=safe,
-        )
-        print kwargs
-        collection.update(update.query.query, update.update_data, **kwargs)
+        self.queue.append(UpdateOp(self.transaction_id, self, update.query.type, safe, update))
+        if self.autoflush:
+            return self.flush()
+
     
     def execute_find_and_modify(self, fm_exp):
+        if self.in_transaction:
+            raise TransactionException('Cannot find and modify in a transaction.')
         self.flush()
         # assert len(fm_exp.update_data) > 0
         collection = self.db[fm_exp.query.type.get_collection_name()]
@@ -272,42 +313,138 @@ class Session(object):
         
         if kwargs['upsert'] and not kwargs.get('new') and len(value) == 0:
             return value
-        
-        return fm_exp.query.type.unwrap(value, fields=fm_exp.query.get_fields())
-    
+
+        # No cache in find and modify, right?  
+        # this is an update operation
+        # obj = self.cache_read(value['_id'])
+        # if obj is not None:
+        #     return obj
+        obj = fm_exp.query.type.unwrap(value, 
+                fields=fm_exp.query.get_fields(),
+                session=self,
+            )
+        if not fm_exp.query.get_fields():
+            self.cache_write(obj)
+        return obj
+
+    @property
+    def transaction_id(self):
+        if not self.transactions:
+            return None
+        return self.transactions[-1]
+
     def get_indexes(self, cls):
         ''' Get the index information for the collection associated with 
         `cls`.  Index information is returned in the same format as *pymongo*.
         '''
         return self.db[cls.get_collection_name()].index_information()
     
-    def clear(self):
+    def clear_queue(self, trans_id=None):
         ''' Clear the queue of database operations without executing any of 
              the pending operations'''
-        self.queue = []
-    
+        if not self.queue:
+            return
+        if trans_id is None:
+            self.queue = []
+            return
+        
+        for index, op in enumerate(self.queue):
+            if op.trans_id == trans_id:
+                break
+        print 'GOT INDEX', index
+        self.queue = self.queue[:index]
+        print '\t', self.queue
+
+    def clear_cache(self):
+        self.cache = {}
+        
     def clear_collection(self, *classes):
         ''' Clear all objects from the collections associated with the 
             objects in `*cls`. **use with caution!**'''
         for c in classes:
-            self.db[c.get_collection_name()].remove()
+            self.queue.append(ClearCollectionOp(self.transaction_id, self, c))
+        if self.autoflush:
+            self.flush()
     
     def flush(self, safe=None):
         ''' Perform all database operations currently in the queue'''
-        if safe is None:
-            safe = self.safe
-        for index, item in enumerate(self.queue):
+        result = None
+        for index, op in enumerate(self.queue):
             try:
-                item.commit(self.db, safe=safe)
+                result = op.execute()
             except:
-                self.clear()
+                self.clear_queue()
+                self.clear_cache()
                 raise
-        self.clear()
-            
+        self.clear_queue()
+        return result
+
+    def dereference(self, ref, allow_none=False):
+        if isinstance(ref, Document):
+            return ref
+        assert hasattr(ref, 'type')
+                
+        obj = self.cache_read(ref.id)
+        if obj is not None:
+            return obj
+        value = self.db.dereference(ref)
+        if value is None and allow_none:
+            obj = None
+            self.cache_write(obj, mongo_id=ref.id)
+        else:
+            obj = ref.type.unwrap(value, session=self)
+            self.cache_write(obj)
+        return obj
+
+    def refresh(self, document):
+        """ Load a new copy of a document from the database.  does not 
+            replace the old one """
+        try:
+            old_cache_size = self.cache_size
+            self.cache_size = 0
+            obj = self.query(type(document)).filter_by(mongo_id=document.mongo_id).one()
+        finally:
+            self.cache_size = old_cache_size
+        self.cache_write(obj)
+        return obj
+
+    def clone(self, document):
+        ''' Serialize a document, remove its _id, and deserialize as a new 
+            object '''
+        
+        wrapped = document.wrap()
+        if '_id' in wrapped:
+            del wrapped['_id']
+        return type(document).unwrap(wrapped, session=self)
     
-    def __enter__(self):
+    def begin_trans(self):
+        self.transactions.append(uuid4())
         return self
-    
+
+    def __enter__(self):
+        return self.begin_trans()
+
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.end()
+        return self.end_trans(exc_type, exc_val, exc_tb)
+
+    def end_trans(self, exc_type=None, exc_val=None, exc_tb=None):
+        # Pop this level of transaction from the stack
+        id = self.transactions.pop()
+
+        # If exception, set us as being in an error state
+        if exc_type:
+            self.clear_queue(trans_id=id)
+
+        # If we aren't at the top level, return
+        if self.transactions:
+            return False
+
+        if not exc_type:
+            self.flush()
+            self.end()
+        else:
+            self.clear_queue()
+            self.clear_cache()
         return False
+
+
