@@ -164,7 +164,7 @@ def test_geo_haystack():
         config_collection_name = 'places'
         loc = GeoField()
         val = IntField()
-        index = Index().geo_haystack('loc', bucket_size=100)
+        index = Index().geo_haystack('loc', bucket_size=100).descending('val')
     s = Session.connect('unit-testing')
     s.clear_collection(Place)
     s.insert(Place(loc=(1,1), val=2))
@@ -241,6 +241,11 @@ def test_nin():
     assert q.nin(T.i, 1, 2, 3).query == {'i' : {'$nin' : [1,2,3]}}, q.nin(T.i, 1, 2, 3).query
     assert q.filter(T.i.nin(1, 2, 3)).query == {'i' : {'$nin' : [1,2,3]}}
 
+def test_exists():
+    q = Query(T, None)
+    assert q.filter(T.i.exists(False)).query == {'i': {'$exists': False}}
+    assert q.filter(T.i.exists(True)).query == {'i': {'$exists': True}}
+
 
 # free-form queries
 
